/*
 * Copyright (c) 2003, Morpho Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 * 3. Neither the name of the Morpho Project nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.morpho.webapp.servlet.jsp.tagext.html;

import java.io.IOException;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;

import jp.morpho.util.ConversionMap;
import jp.morpho.webapp.resource.MessageResourceManager;
import jp.morpho.webapp.servlet.http.HttpActionRequest;
import jp.morpho.webapp.servlet.http.HttpActionRequestFactory;
import jp.morpho.webapp.servlet.http.HttpSessionAttributeNames;
import jp.morpho.webapp.servlet.jsp.tagext.TagManager;
import jp.morpho.webapp.servlet.jsp.tagext.TagTemplate;

/**
 * <P>
 * HTML  INPUT vf^OnhłB
 * eLXǵA`FbN{bNXAWI{^Ȃǂ̃Rg[ɓK^Onh쐬邽߂ɁATuNXOƂ abstract NX񋟂܂B
 * </P>
 * <P>
 * INPUT  HTML 4 `͎̂ƂłB
 * <PRE>
 * &lt;!ENTITY % InputType
 *   "(TEXT | PASSWORD | CHECKBOX |
 *     RADIO | SUBMIT | RESET |
 *     FILE | HIDDEN | IMAGE | BUTTON)"
 *    &gt;
 * 
 * &lt;!-- attribute name required for all but submit and reset --&gt;
 * &lt;!ELEMENT INPUT - O EMPTY              -- form control --&gt;
 * &lt;!ATTLIST INPUT
 *   %attrs;                              -- %coreattrs, %i18n, %events --
 *   type        %InputType;    TEXT      -- what kind of widget is needed --
 *   name        CDATA          #IMPLIED  -- submit as part of form --
 *   value       CDATA          #IMPLIED  -- Specify for radio buttons and checkboxes --
 *   checked     (checked)      #IMPLIED  -- for radio buttons and check boxes --
 *   disabled    (disabled)     #IMPLIED  -- unavailable in this context --
 *   readonly    (readonly)     #IMPLIED  -- for text and passwd --
 *   size        CDATA          #IMPLIED  -- specific to each type of field --
 *   maxlength   NUMBER         #IMPLIED  -- max chars for text fields --
 *   src         %URI;          #IMPLIED  -- for fields with images --
 *   alt         CDATA          #IMPLIED  -- short description --
 *   usemap      %URI;          #IMPLIED  -- use client-side image map --
 *   ismap       (ismap)        #IMPLIED  -- use server-side image map --
 *   tabindex    NUMBER         #IMPLIED  -- position in tabbing order --
 *   accesskey   %Character;    #IMPLIED  -- accessibility key character --
 *   onfocus     %Script;       #IMPLIED  -- the element got the focus --
 *   onblur      %Script;       #IMPLIED  -- the element lost the focus --
 *   onselect    %Script;       #IMPLIED  -- some text was selected --
 *   onchange    %Script;       #IMPLIED  -- the element value was changed --
 *   accept      %ContentTypes; #IMPLIED  -- list of MIME types for file upload --
 *   &gt;
 * </PRE>
 * </P>
 * @author Kumiko Hiroi
 */
public abstract class InputTag
	extends ControlTagSupport
	implements TagTemplate
{

	/**
	 * <P>
	 * InputTag 쐬܂B
	 * </P>
	 */
	public InputTag()
	{
		super();
	}

	/**
	 * <P>
	 * accept Ԃ܂B
	 * </P>
	 * @return accept 
	 */
	public String getAccept()
	{
		return getAttributeMap().getString("accept");
	}

	/**
	 * <P>
	 * accesskey Ԃ܂B
	 * </P>
	 * @return accesskey 
	 */
	public String getAccesskey()
	{
		return getAttributeMap().getString("accesskey");
	}

	/**
	 * <P>
	 * alt Ԃ܂B
	 * </P>
	 * @return alt 
	 */
	public String getAlt()
	{
		return getAttributeMap().getString("alt");
	}

	/**
	 * <P>
	 * checked Ԃ܂B
	 * </P>
	 * @return checked 
	 */
	public boolean getChecked()
	{
		return getAttributeMap().getBoolean("checked");
	}

	/**
	 * <P>
	 * disabled Ԃ܂B
	 * </P>
	 * @return disabled 
	 */
	public boolean getDisabled()
	{
		return getAttributeMap().getBoolean("disabled");
	}

	/**
	 * <P>
	 * ismap Ԃ܂B
	 * </P>
	 * @return ismap 
	 */
	public boolean getIsmap()
	{
		return getAttributeMap().getBoolean("ismap");
	}

	/**
	 * <P>
	 * maxlength Ԃ܂B
	 * </P>
	 * @return maxlength 
	 */
	public String getMaxlength()
	{
		return getAttributeMap().getString("maxlength");
	}

	/**
	 * <P>
	 * name Ԃ܂B
	 * </P>
	 * @return name 
	 */
	public String getName()
	{
		return getAttributeMap().getString("name");
	}

	/**
	 * <P>
	 * onblur Ԃ܂B
	 * </P>
	 * @return onblur 
	 */
	public String getOnblur()
	{
		return getAttributeMap().getString("onblur");
	}

	/**
	 * <P>
	 * onchange Ԃ܂B
	 * </P>
	 * @return onchange 
	 */
	public String getOnchange()
	{
		return getAttributeMap().getString("onchange");
	}

	/**
	 * <P>
	 * onfocus Ԃ܂B
	 * </P>
	 * @return onfocus 
	 */
	public String getOnfocus()
	{
		return getAttributeMap().getString("onfocus");
	}

	/**
	 * <P>
	 * onselect Ԃ܂B
	 * </P>
	 * @return onselect 
	 */
	public String getOnselect()
	{
		return getAttributeMap().getString("onselect");
	}

	/**
	 * <P>
	 * readonly Ԃ܂B
	 * </P>
	 * @return readonly 
	 */
	public boolean getReadonly()
	{
		return getAttributeMap().getBoolean("readonly");
	}

	/**
	 * <P>
	 * size Ԃ܂B
	 * </P>
	 * @return size 
	 */
	public String getSize()
	{
		return getAttributeMap().getString("size");
	}

	/**
	 * <P>
	 * src Ԃ܂B
	 * </P>
	 * @return src 
	 */
	public String getSrc()
	{
		return getAttributeMap().getString("src");
	}

	/**
	 * <P>
	 * tabindex Ԃ܂B
	 * </P>
	 * @return tabindex 
	 */
	public String getTabindex()
	{
		return getAttributeMap().getString("tabindex");
	}

	/**
	 * <P>
	 * type Ԃ܂B
	 * </P>
	 * @return type 
	 */
	public String getType()
	{
		return getAttributeMap().getString("type");
	}

	/**
	 * <P>
	 * usemap Ԃ܂B
	 * </P>
	 * @return usemap 
	 */
	public String getUsemap()
	{
		return getAttributeMap().getString("usemap");
	}

	/**
	 * <P>
	 * value Ԃ܂B
	 * </P>
	 * @return value 
	 */
	public String getValue()
	{
		return getAttributeMap().getString("value");
	}

	/**
	 * <P>
	 * template Ԃ܂B
	 * </P>
	 * @return template 
	 */
	public String getTemplate()
	{
		return getAttributeMap().getString("template");
	}

	/**
	 * <P>
	 * accept ݒ肵܂B
	 * </P>
	 * @param accept accept 
	 */
	public void setAccept(String accept)
	{
		getAttributeMap().putString("accept", accept);
	}

	/**
	 * <P>
	 * accesskey ݒ肵܂B
	 * </P>
	 * @param accesskey accesskey 
	 */
	public void setAccesskey(String accesskey)
	{
		getAttributeMap().putString("accesskey", accesskey);
	}

	/**
	 * <P>
	 * alt ݒ肵܂B
	 * </P>
	 * @param alt alt 
	 */
	public void setAlt(String alt)
	{
		getAttributeMap().putString("alt", alt);
	}

	/**
	 * <P>
	 * checked ݒ肵܂B
	 * </P>
	 * @param checked checked 
	 */
	public void setChecked(boolean checked)
	{
		getAttributeMap().putBoolean("checked", checked);
	}

	/**
	 * <P>
	 * disabled ݒ肵܂B
	 * </P>
	 * @param disabled disabled 
	 */
	public void setDisabled(boolean disabled)
	{
		getAttributeMap().putBoolean("disabled", disabled);
	}

	/**
	 * <P>
	 * ismap ݒ肵܂B
	 * </P>
	 * @param ismap ismap 
	 */
	public void setIsmap(boolean ismap)
	{
		getAttributeMap().putBoolean("ismap", ismap);
	}

	/**
	 * <P>
	 * maxlength ݒ肵܂B
	 * </P>
	 * @param maxlength maxlength 
	 */
	public void setMaxlength(String maxlength)
	{
		getAttributeMap().putString("maxlength", maxlength);
	}

	/**
	 * <P>
	 * name ݒ肵܂B
	 * </P>
	 * @param name name 
	 */
	public void setName(String name)
	{
		getAttributeMap().putString("name", name);
	}

	/**
	 * <P>
	 * onblur ݒ肵܂B
	 * </P>
	 * @param onblur onblur 
	 */
	public void setOnblur(String onblur)
	{
		getAttributeMap().putString("onblur", onblur);
	}

	/**
	 * <P>
	 * onchange ݒ肵܂B
	 * </P>
	 * @param onchange onchange 
	 */
	public void setOnchange(String onchange)
	{
		getAttributeMap().putString("onchange", onchange);
	}

	/**
	 * <P>
	 * onfocus ݒ肵܂B
	 * </P>
	 * @param onfocus onfocus 
	 */
	public void setOnfocus(String onfocus)
	{
		getAttributeMap().putString("onfocus", onfocus);
	}

	/**
	 * <P>
	 * onselect ݒ肵܂B
	 * </P>
	 * @param onselect onselect 
	 */
	public void setOnselect(String onselect)
	{
		getAttributeMap().putString("onselect", onselect);
	}

	/**
	 * <P>
	 * readonly ݒ肵܂B
	 * </P>
	 * @param readonly readonly 
	 */
	public void setReadonly(boolean readonly)
	{
		getAttributeMap().putBoolean("readonly", readonly);
	}

	/**
	 * <P>
	 * size ݒ肵܂B
	 * </P>
	 * @param size size 
	 */
	public void setSize(String size)
	{
		getAttributeMap().putString("size", size);
	}

	/**
	 * <P>
	 * src ݒ肵܂B
	 * </P>
	 * @param src src 
	 */
	public void setSrc(String src)
	{
		getAttributeMap().putString("src", src);
	}

	/**
	 * <P>
	 * tabindex ݒ肵܂B
	 * </P>
	 * @param tabindex tabindex 
	 */
	public void setTabindex(String tabindex)
	{
		getAttributeMap().putString("tabindex", tabindex);
	}

	/**
	 * <P>
	 * type ݒ肵܂B
	 * </P>
	 * @param type type 
	 */
	public void setType(String type)
	{
		getAttributeMap().putString("type", type);
	}

	/**
	 * <P>
	 * usemap ݒ肵܂B
	 * </P>
	 * @param usemap usemap 
	 */
	public void setUsemap(String usemap)
	{
		getAttributeMap().putString("usemap", usemap);
	}

	/**
	 * <P>
	 * value ݒ肵܂B
	 * </P>
	 * @param value value 
	 */
	public void setValue(String value)
	{
		getAttributeMap().putString("value", value);
	}

	/**
	 * <P>
	 * template ݒ肵܂B
	 * </P>
	 * @param template template 
	 */
	public void setTemplate(String template)
	{
		getAttributeMap().putString("template", template);
	}

	/**
	 * <P>
	 * Jn^ȌłB
	 * </P>
	 * @return EVAL_BODY_BUFFERED
	 * @see javax.servlet.jsp.tagext.Tag#doStartTag()
	 */
	public int doStartTag() throws JspException
	{
		//INPUT^O쐬܂
		StringBuffer buf = new StringBuffer();
		buf.append("<input");

		//TagTemplate擾܂
		TagManager tm = this.getTagManager();
		TagTemplate ttmp = null;
		if (this.getAttributeMap().containsKey("template"))
		{
			ttmp = tm.getTagTemplate(this.getTagType(), this.getTemplate());
		}
		else
		{
			ttmp = tm.getTagTemplate(this.getTagType(), this.getName());
		}

		ConversionMap attr = new ConversionMap();
		if (ttmp != null)
		{
			attr.putAll(ttmp.getAttributeMap());
		}
		attr.putAll(this.getAttributeMap());

		String keys[] = new String[attr.size()];
		keys = (String[])attr.keySet().toArray(keys);
		for (int i = 0; i < keys.length; i++)
		{
			if (keys[i].equals("template"))
			{
				//Ȃ
			}
			else if (keys[i].equals("disabled") ||
					keys[i].equals("readonly") ||
					keys[i].equals("checked") ||
					keys[i].equals("ismap"))
			{
				if (attr.getBoolean(keys[i]))
				{
					buf.append(" ").append(keys[i]);
				}
			}
			else
			{
				buf.append(" ").append(keys[i]).append("=\"").append(attr.getString(keys[i])).append("\"");
			}
		}
		buf.append(">");

		try
		{
			JspWriter out = pageContext.getOut();
			out.print(buf.toString());
		}
		catch (IOException e)
		{
			throw new JspException(e.getMessage());
		}

		return super.doStartTag();
	}

	/**
	 * <P>
	 * ̃^Õ^CvԂ܂B
	 * </P>
	 * @return ̃^Õ^Cv
	 * @see jp.morpho.webapp.servlet.jsp.tagext.TagTemplate#getTagType()
	 */
	public abstract String getTagType();

	/**
	 * <P>
	 * checked ݒ肳ȂꍇAname ɐݒ肳Ă閼OL[ɂ HTTP p[^l擾Achecked ݒ肵܂B
	 * name ݒ肳ĂȂꍇ͉܂B
	 * </P>
	 */
	protected void setCheckedByFormData()
	{
		if (!this.getAttributeMap().containsKey("checked") && this.getAttributeMap().containsKey("name"))
		{
			HttpActionRequest request =
				HttpActionRequestFactory.getInstance().getRequest(
					(HttpServletRequest)pageContext.getRequest());
			ConversionMap param = ConversionMap.valueOf(request.getParameterMap());
			if (!param.isNull(this.getName()))
			{
				if (param.isArray(this.getName()))
				{
					String values[] = (String[])param.getObject(this.getName());
					String val = this.getValue();
					for (int i = 0; i < values.length; i++)
					{
						if (val.equals(values[i]))
						{
							this.setChecked(true);
							break;
						}
					}
				}
				else
				{
					if (this.getValue().equals(param.getString(this.getName())))
					{
						this.setChecked(true);
					}
				}
			}
		}
	}

	/**
	 * <P>
	 * value ݒ肳ȂꍇAname ɐݒ肳Ă閼OL[ɂ HTTP p[^l擾Avalue ɐݒ肵܂B
	 * name ݒ肳ĂȂꍇ͉܂B
	 * </P>
	 */
	protected void setValueByFormData() throws JspException
	{
		if (!this.getAttributeMap().containsKey("value") && this.getAttributeMap().containsKey("name"))
		{
			HttpActionRequest request =
				HttpActionRequestFactory.getInstance().getRequest(
					(HttpServletRequest)pageContext.getRequest());
			ConversionMap param = ConversionMap.valueOf(request.getParameterMap());
			HtmlEncoder encoder = this.getTagManager().getHtmlEncoder();
			if (param.containsKey(this.getName()))
			{
				this.getAttributeMap().putString("value", encoder.encode(param.getString(this.getName())));
			}
		}
	}

	/**
	 * <P>
	 * value ݒ肳ȂꍇAname ɐݒ肳Ă閼OL[ɂăbZ[W \[Xl擾Avalue ɐݒ肵܂B
	 * name ݒ肳ĂȂꍇ͉܂B
	 * </P>
	 */
	protected void setValueByMessageResource() throws JspException
	{
		if (!this.getAttributeMap().containsKey("value") && this.getAttributeMap().containsKey("name"))
		{
			HttpServletRequest request = (HttpServletRequest)pageContext.getRequest();
			Locale locale = (Locale)request.getSession().getAttribute(HttpSessionAttributeNames.locale);
			if (locale == null)
			{
				locale = request.getLocale();
			}

			MessageResourceManager mrm = this.getMessageResourceManager();
			try
			{
				this.setValue(mrm.getFieldName(this.getName(), locale));
			}
			catch (Exception e)
			{
			}
		}
	}

}
